@@ -1,22 +1,24 @@
 import numpy as np
 import warnings
 from .common import Benchmark, safe_import
+from asv_runner.benchmarks.mark import SkipNotImplemented
 
 with safe_import():
-    from scipy.linalg import (eigh,
-                              cholesky_banded, cho_solve_banded, eig_banded)
+    from scipy.linalg import eigh, cholesky_banded, cho_solve_banded, eig_banded
     from scipy.sparse.linalg import lobpcg, eigsh, LinearOperator
-    from scipy.sparse.linalg._special_sparse_arrays import (Sakurai,
-                                                            MikotaPair)
-
-# ensure that we are benchmarking a consistent outcome;
-# (e.g. if the code wasn't able to find a solution at all,
-# the timing of the benchmark would become useless).
-msg = ("the benchmark code did not converge as expected, "
-       "the timing is therefore useless")
+    from scipy.sparse.linalg._special_sparse_arrays import Sakurai, MikotaPair
 
 
 class Bench(Benchmark):
+    # ensure that we are benchmarking a consistent outcome;
+    # (e.g. if the code wasn't able to find a solution accurately
+    # enough the timing of the benchmark would become useless).
+    # if no convergence sufficiently for timing to be relevant;
+    # ensure that the unconverged benchmark fails by sleeping
+    # over the timeout limit.
+    # all the benchmark tests are repeated in the unit test suite
+    # `scipy/sparse/linalg/_eigen/lobpcg/tests/test_lobpcg.py`
+
     params = [
         [],
         ['lobpcg', 'eigsh', 'lapack']
@@ -51,7 +53,7 @@ class Bench(Benchmark):
 
         if solver == 'lapack' and n > 512:
             # skip: slow, and not useful to benchmark
-            raise NotImplementedError()
+            raise SkipNotImplemented(f"{solver} too slow to benchmark with {n=}.")
 
     def setup_sakurai(self, n, solver):
         self.shape = (n, n)
@@ -65,7 +67,7 @@ class Bench(Benchmark):
         sakurai_obj = Sakurai(n)
         self.A = sakurai_obj.tobanded().astype(np.float64)
         self.eigenvalues = sakurai_obj.eigenvalues
-    
+
     def time_mikota(self, n, solver):
         def a(x):
             return cho_solve_banded((c, False), x)
@@ -79,14 +81,13 @@ class Bench(Benchmark):
             # `lobpcg` allows callable parameters `Ac` and `Bc` directly
             # `lobpcg` solves ``Ax = lambda Bx`` and applies here a preconditioner
             # given by the matrix inverse in `np.float32` of 'Ab` that itself
-            # is `np.float64`. 
+            # is `np.float64`.
             c = cholesky_banded(self.Ab.astype(np.float32))
             with warnings.catch_warnings():
                 warnings.simplefilter("ignore")
                 el, _ = lobpcg(self.Ac, X, self.Bc, M=a, tol=1e-4,
                                maxiter=40, largest=False)
             accuracy = max(abs(ee - el) / ee)
-            assert accuracy < tol, msg
         elif solver == 'eigsh':
             # `eigsh` ARPACK here is called on ``Bx = 1/lambda Ax``
             # to get fast convergence speed similar to that of `lobpcg` above
@@ -101,14 +102,16 @@ class Bench(Benchmark):
             ea, _ = eigsh(B, k=m, M=A, Minv=a_l, which='LA', tol=1e-4, maxiter=50,
                           v0 = rng.normal(size=(n, 1)))
             accuracy = max(abs(ee - np.sort(1./ea)) / ee)
-            assert accuracy < tol, msg
         else:
             # `eigh` is the only dense eigensolver for generalized eigenproblems
             # ``Ax = lambda Bx`` and needs both matrices as dense arrays
             # making it very slow for large matrix sizes
             ed, _ = eigh(self.Aa, self.Ba, subset_by_index=(0, m - 1))
             accuracy = max(abs(ee - ed) / ee)
-            assert accuracy < tol, msg
+
+        if accuracy > tol:
+            # not convergence sufficiently for timing to be relevant
+            raise SkipNotImplemented("Insufficient accuracy achieved")
 
     def time_sakurai(self, n, solver):
         # the Sakurai matrix ``A`` is ill-conditioned so convergence of both
@@ -125,17 +128,18 @@ class Bench(Benchmark):
                 warnings.simplefilter("ignore")
                 el, _ = lobpcg(self.A, X, tol=1e-9, maxiter=5000, largest=False)
             accuracy = max(abs(ee - el) / ee)
-            assert accuracy < tol, msg
         elif solver == 'eigsh':
             a_l = LinearOperator((n, n), matvec=self.A, matmat=self.A, dtype='float64')
             ea, _ = eigsh(a_l, k=m, which='SA', tol=1e-9, maxiter=15000,
                           v0 = rng.normal(size=(n, 1)))
             accuracy = max(abs(ee - ea) / ee)
-            assert accuracy < tol, msg
         else:
             ed, _ = eigh(self.Aa, subset_by_index=(0, m - 1))
             accuracy = max(abs(ee - ed) / ee)
-            assert accuracy < tol, msg
+
+        if accuracy > tol:
+            # not convergence sufficiently for timing to be relevant
+            raise SkipNotImplemented("Insufficient accuracy achieved")
 
     def time_sakurai_inverse(self, n, solver):
         # apply inverse iterations in  `lobpcg` and `eigsh` ARPACK
@@ -154,15 +158,16 @@ class Bench(Benchmark):
                 warnings.simplefilter("ignore")
                 el, _ = lobpcg(a, X, tol=1e-9, maxiter=8)
             accuracy = max(abs(ee - 1. / el) / ee)
-            assert accuracy < tol, msg
         elif solver == 'eigsh':
             c = cholesky_banded(self.A)
             a_l = LinearOperator((n, n), matvec=a, matmat=a, dtype='float64')
             ea, _ = eigsh(a_l, k=m, which='LA', tol=1e-9, maxiter=8,
                           v0 = rng.normal(size=(n, 1)))
             accuracy = max(abs(ee - np.sort(1. / ea)) / ee)
-            assert accuracy < tol, msg
         else:
             ed, _ = eig_banded(self.A, select='i', select_range=[0, m-1])
             accuracy = max(abs(ee - ed) / ee)
-            assert accuracy < tol, msg
+
+        if accuracy > tol:
+            # not convergence sufficiently for timing to be relevant
+            raise SkipNotImplemented("Insufficient accuracy achieved")
