@@ -63,5 +63,16 @@ static struct PyModuleDef moduledef = {
 PyMODINIT_FUNC
 PyInit__fpumode(void)
 {
-    return PyModule_Create(&moduledef);
+    PyObject *module;
+
+    module = PyModule_Create(&moduledef);
+    if (module == NULL) {
+        return module;
+    }
+
+#if Py_GIL_DISABLED
+    PyUnstable_Module_SetGIL(module, Py_MOD_GIL_NOT_USED);
+#endif
+
+    return module;
 }
