@@ -10,7 +10,7 @@ from scipy._lib._array_api import (
     array_namespace,
     xp_size,
     xp_copy,
-    xp_broadcast_promote
+    xp_promote
 )
 from scipy._lib._util import MapWrapper
 
@@ -323,7 +323,8 @@ def cubature(f, a, b, *, rule="gk21", rtol=1e-8, atol=0, max_subdivisions=10000,
 
     # Convert a and b to arrays and convert each point in points to an array, promoting
     # each to a common floating dtype.
-    a, b, *points = xp_broadcast_promote(a, b, *points, force_floating=True)
+    a, b, *points = xp_promote(a, b, *points, broadcast=True, force_floating=True,
+                               xp=xp)
     result_dtype = a.dtype
 
     if xp_size(a) == 0 or xp_size(b) == 0:
