@@ -34,7 +34,7 @@ __all__ = ["AAA", "FloaterHormannInterpolator"]
 
 
 class _BarycentricRational:
-    """Base class for Barycentric representation of a rational function."""
+    """Base class for barycentric representation of a rational function."""
     def __init__(self, x, y, **kwargs):
         # input validation
         z = np.asarray(x)
@@ -399,7 +399,7 @@ class AAA(_BarycentricRational):
     >>> with warnings.catch_warnings():
     ...     warnings.simplefilter('ignore', RuntimeWarning)
     ...     r.clean_up()
-    4
+    4  # may vary
     >>> mask = np.abs(r.residues()) < 1e-13
     >>> axs[1].plot(r.poles().real[~mask], r.poles().imag[~mask], '.')
     >>> axs[1].plot(r.poles().real[mask], r.poles().imag[mask], 'r.')
@@ -597,11 +597,10 @@ class AAA(_BarycentricRational):
 
 
 class FloaterHormannInterpolator(_BarycentricRational):
-    r"""
-    Floater-Hormann barycentric rational interpolation.
+    r"""Floater-Hormann barycentric rational interpolator (C∞ smooth on real axis).
 
     As described in [1]_, the method of Floater and Hormann computes weights for a
-    Barycentric rational interpolant with no poles on the real axis.
+    barycentric rational interpolant with no poles on the real axis.
 
     Parameters
     ----------
@@ -610,11 +609,11 @@ class FloaterHormannInterpolator(_BarycentricRational):
         complex but must be finite.
     y : array_like, shape (n, ...)
         Array containing values of the dependent variable. Infinite and NaN values
-        of `values` and corresponding values of `x` will be discarded.
-    d : int, optional
-        Blends ``n - d`` degree `d` polynomials together. For ``d = n - 1`` it is
-        equivalent to polynomial interpolation. Must satisfy ``0 <= d < n``,
-        defaults to 3.
+        of `y` and corresponding values of `x` will be discarded.
+    d : int, default: 3
+        Integer satisfying ``0 <= d < n``. Floater-Hormann interpolation blends
+        ``n - d`` polynomials of degree `d` together; for ``d = n - 1``, this is
+        equivalent to polynomial interpolation.
 
     Attributes
     ----------
@@ -639,8 +638,8 @@ class FloaterHormannInterpolator(_BarycentricRational):
         r(x) = \frac{\sum_{i=0}^{n-d} \lambda_i(x) p_i(x)}
         {\sum_{i=0}^{n-d} \lambda_i(x)},
 
-    where :math:`p_i(x)` is an interpolating polynomials of at most degree `d` through
-    the points :math:`(x_i,y_i),\dots,(x_{i+d},y_{i+d}), and :math:`\lambda_i(z)` are
+    where :math:`p_i(x)` is an interpolating polynomial of at most degree `d` through
+    the points :math:`(x_i,y_i),\dots,(x_{i+d},y_{i+d})`, and :math:`\lambda_i(z)` are
     blending functions defined by
 
     .. math::
@@ -649,8 +648,8 @@ class FloaterHormannInterpolator(_BarycentricRational):
 
     When ``d = n - 1`` this reduces to polynomial interpolation.
 
-    Due to its stability following barycentric representation of the above equation
-    is used instead for computation
+    Due to its stability, the following barycentric representation of the above equation
+    is used for computation
 
     .. math::
 
@@ -680,16 +679,17 @@ class FloaterHormannInterpolator(_BarycentricRational):
     >>> import numpy as np
     >>> from scipy.interpolate import (FloaterHormannInterpolator,
     ...                                BarycentricInterpolator)
-    >>> def f(z):
-    ...     return 1/(1 + z**2)
-    >>> z = np.linspace(-5, 5, num=15)
-    >>> r = FloaterHormannInterpolator(z, f(z))
-    >>> p = BarycentricInterpolator(z, f(z))
-    >>> zz = np.linspace(-5, 5, num=1000)
+    >>> def f(x):
+    ...     return 1/(1 + x**2)
+    >>> x = np.linspace(-5, 5, num=15)
+    >>> r = FloaterHormannInterpolator(x, f(x))
+    >>> p = BarycentricInterpolator(x, f(x))
+    >>> xx = np.linspace(-5, 5, num=1000)
     >>> import matplotlib.pyplot as plt
     >>> fig, ax = plt.subplots()
-    >>> ax.plot(zz, r(zz), label="Floater=Hormann")
-    >>> ax.plot(zz, p(zz), label="Polynomial")
+    >>> ax.plot(xx, f(xx), label="f(x)")
+    >>> ax.plot(xx, r(xx), "--", label="Floater-Hormann")
+    >>> ax.plot(xx, p(xx), "--", label="Polynomial")
     >>> ax.legend()
     >>> plt.show()
     """
