@@ -2,9 +2,9 @@ from numpy.testing import assert_array_almost_equal, assert_allclose, assert_
 from numpy import (array, eye, zeros, empty_like, empty, tril_indices_from,
                    tril, triu_indices_from, spacing, float32, float64,
                    complex64, complex128)
+from numpy.exceptions import ComplexWarning
 from numpy.random import rand, randint, seed
 from scipy.linalg import ldl
-from scipy._lib._util import ComplexWarning
 import pytest
 from pytest import raises as assert_raises, warns
 
