@@ -96,14 +96,14 @@ def dump_dataset(filename, data):
     fid = open(filename, 'w')
     try:
         for line in data:
-            fid.write("%s\n" % " ".join(line))
+            fid.write(f"{' '.join(line)}\n")
     finally:
         fid.close()
 
 
 def dump_datasets(filename):
     base, ext = os.path.splitext(os.path.basename(filename))
-    base += '_%s' % ext[1:]
+    base += f'_{ext[1:]}'
     datadir = os.path.join(DATA_DIR, base)
     os.makedirs(datadir)
     datasets = parse_ipp_file(filename)
