@@ -1,54 +1,54 @@
 #include "xsf_wrappers.h"
-#include "xsf/airy.h"
-#include "xsf/alg.h"
-#include "xsf/amos.h"
-#include "xsf/bessel.h"
-#include "xsf/beta.h"
-#include "xsf/binom.h"
-#include "xsf/cdflib.h"
-#include "xsf/digamma.h"
-#include "xsf/ellip.h"
-#include "xsf/expint.h"
-#include "xsf/fresnel.h"
-#include "xsf/gamma.h"
-#include "xsf/hyp2f1.h"
-#include "xsf/kelvin.h"
-#include "xsf/lambertw.h"
-#include "xsf/log_exp.h"
-#include "xsf/loggamma.h"
-#include "xsf/mathieu.h"
-#include "xsf/par_cyl.h"
-#include "xsf/sici.h"
-#include "xsf/specfun.h"
-#include "xsf/sph_bessel.h"
-#include "xsf/sph_harm.h"
-#include "xsf/sphd_wave.h"
-#include "xsf/stats.h"
-#include "xsf/struve.h"
-#include "xsf/trig.h"
-#include "xsf/wright_bessel.h"
-#include "xsf/zeta.h"
+#include <xsf/airy.h>
+#include <xsf/amos.h>
+#include <xsf/bessel.h>
+#include <xsf/beta.h>
+#include <xsf/binom.h>
+#include <xsf/cdflib.h>
+#include <xsf/digamma.h>
+#include <xsf/ellip.h>
+#include <xsf/erf.h>
+#include <xsf/exp.h>
+#include <xsf/expint.h>
+#include <xsf/fresnel.h>
+#include <xsf/gamma.h>
+#include <xsf/hyp2f1.h>
+#include <xsf/kelvin.h>
+#include <xsf/lambertw.h>
+#include <xsf/log.h>
+#include <xsf/log_exp.h>
+#include <xsf/loggamma.h>
+#include <xsf/mathieu.h>
+#include <xsf/par_cyl.h>
+#include <xsf/sici.h>
+#include <xsf/specfun.h>
+#include <xsf/sph_bessel.h>
+#include <xsf/sph_harm.h>
+#include <xsf/sphd_wave.h>
+#include <xsf/stats.h>
+#include <xsf/struve.h>
+#include <xsf/trig.h>
+#include <xsf/wright_bessel.h>
+#include <xsf/zeta.h>
 #include "xsf_special.h"
 
-#include "xsf/cephes/cbrt.h"
-#include "xsf/cephes/erfinv.h"
-#include "xsf/cephes/exp10.h"
-#include "xsf/cephes/exp2.h"
-#include "xsf/cephes/expn.h"
-#include "xsf/cephes/fresnl.h"
-#include "xsf/cephes/hyperg.h"
-#include "xsf/cephes/igam.h"
-#include "xsf/cephes/igami.h"
-#include "xsf/cephes/jv.h"
-#include "xsf/cephes/lanczos.h"
-#include "xsf/cephes/poch.h"
-#include "xsf/cephes/rgamma.h"
-#include "xsf/cephes/round.h"
-#include "xsf/cephes/scipy_iv.h"
-#include "xsf/cephes/spence.h"
-#include "xsf/cephes/trig.h"
-#include "xsf/cephes/unity.h"
-#include "xsf/cephes/yn.h"
+#include <xsf/cephes/cbrt.h>
+#include <xsf/cephes/erfinv.h>
+#include <xsf/cephes/expn.h>
+#include <xsf/cephes/fresnl.h>
+#include <xsf/cephes/hyperg.h>
+#include <xsf/cephes/igam.h>
+#include <xsf/cephes/igami.h>
+#include <xsf/cephes/jv.h>
+#include <xsf/cephes/lanczos.h>
+#include <xsf/cephes/poch.h>
+#include <xsf/cephes/rgamma.h>
+#include <xsf/cephes/round.h>
+#include <xsf/cephes/scipy_iv.h>
+#include <xsf/cephes/spence.h>
+#include <xsf/cephes/trig.h>
+#include <xsf/cephes/unity.h>
+#include <xsf/cephes/yn.h>
 
 using namespace std;
 
@@ -60,7 +60,7 @@ npy_cdouble to_ccomplex(complex<double> z) { return {z.real(), z.imag()}; }
 
 } // namespace
 
-npy_cdouble chyp1f1_wrap(double a, double b, npy_cdouble z) { return to_ccomplex(xsf::chyp1f1(a, b, to_complex(z))); }
+npy_cdouble chyp1f1_wrap(double a, double b, npy_cdouble z) { return to_ccomplex(xsf::hyp1f1(a, b, to_complex(z))); }
 
 double hypU_wrap(double a, double b, double x) { return xsf::hypu(a, b, x); }
 
@@ -309,20 +309,17 @@ int cephes_ellpj_wrap(double u, double m, double *sn, double *cn, double *dn, do
     return xsf::cephes::ellpj(u, m, sn, cn, dn, ph);
 }
 
-int xsf_sici(double x, double *si, double *ci) { return xsf::sici(x, si, ci); }
+int xsf_sici(double x, double *si, double *ci) { return xsf::sici(x, *si, *ci); }
 
-int xsf_shichi(double x, double *si, double *ci) { return xsf::shichi(x, si, ci); }
+int xsf_shichi(double x, double *si, double *ci) { return xsf::shichi(x, *si, *ci); }
 
 int xsf_csici(npy_cdouble x, npy_cdouble *si, npy_cdouble *ci) {
-    return xsf::sici(to_complex(x),
-		     reinterpret_cast<complex<double> *>(si),
-		     reinterpret_cast<complex<double> *>(ci));
+    return xsf::sici(to_complex(x), *reinterpret_cast<complex<double> *>(si), *reinterpret_cast<complex<double> *>(ci));
 }
 
 int xsf_cshichi(npy_cdouble x, npy_cdouble *shi, npy_cdouble *chi) {
-    return xsf::shichi(to_complex(x),
-		       reinterpret_cast<complex<double> *>(shi),
-		       reinterpret_cast<complex<double> *>(chi));
+    return xsf::shichi(to_complex(x), *reinterpret_cast<complex<double> *>(shi),
+                       *reinterpret_cast<complex<double> *>(chi));
 }
 
 double cephes__struve_asymp_large_z(double v, double z, Py_ssize_t is_h, double *err) {
@@ -354,7 +351,7 @@ double xsf_beta(double a, double b) { return xsf::beta(a, b); }
 
 double xsf_betaln(double a, double b) { return xsf::betaln(a, b); }
 
-double xsf_cbrt(double x) { return xsf::cbrt(x); }
+double xsf_cbrt(double x) { return xsf::cephes::cbrt(x); }
 
 double xsf_gamma(double x) { return xsf::gamma(x); }
 
@@ -382,28 +379,42 @@ double cephes_igam_fac(double a, double x) { return xsf::cephes::detail::igam_fa
 
 double cephes_lanczos_sum_expg_scaled(double x) { return xsf::cephes::lanczos_sum_expg_scaled(x); }
 
-double cephes_erf(double x) { return xsf::cephes::erf(x); }
-
-double cephes_erfc(double x) { return xsf::cephes::erfc(x); }
-
 double cephes_poch(double x, double m) { return xsf::cephes::poch(x, m); }
 
 double cephes_rgamma(double x) { return xsf::cephes::rgamma(x); }
 
 double xsf_zetac(double x) { return xsf::zetac(x); }
 
-double cephes_log1p(double x) { return xsf::cephes::log1p(x); }
-
-double cephes_log1pmx(double x) { return xsf::cephes::log1pmx(x); }
-
 double cephes_lgam1p(double x) { return xsf::cephes::lgam1p(x); }
 
-double cephes_expm1(double x) { return xsf::cephes::expm1(x); }
-
 double cephes_expn(int n, double x) { return xsf::cephes::expn(n, x); }
 
 double xsf_ellipe(double x) { return xsf::ellipe(x); }
 
+double xsf_erf(double x) { return xsf::erf(x); }
+
+npy_cdouble xsf_cerf(npy_cdouble z) { return to_ccomplex(xsf::erf(to_complex(z))); }
+
+double xsf_erfc(double x) { return xsf::erfc(x); }
+
+npy_cdouble xsf_cerfc(npy_cdouble z) { return to_ccomplex(xsf::erfc(to_complex(z))); }
+
+double xsf_erfcx(double x) { return xsf::erfcx(x); }
+
+npy_cdouble xsf_cerfcx(npy_cdouble z) { return to_ccomplex(xsf::erfcx(to_complex(z))); }
+
+double xsf_dawsn(double x) { return xsf::dawsn(x); }
+
+npy_cdouble xsf_cdawsn(npy_cdouble z) { return to_ccomplex(xsf::dawsn(to_complex(z))); }
+
+double xsf_erfi(double x) { return xsf::erfi(x); }
+
+npy_cdouble xsf_cerfi(npy_cdouble z) { return to_ccomplex(xsf::erfi(to_complex(z))); }
+
+npy_cdouble xsf_cwofz(npy_cdouble z) { return to_ccomplex(xsf::wofz(to_complex(z))); }
+
+double xsf_voigt_profile(double x, double sigma, double gamma) { return xsf::voigt_profile(x, sigma, gamma); }
+
 double cephes_ellpk(double x) { return xsf::ellipkm1(x); }
 
 double cephes_ellie(double phi, double m) { return xsf::ellipeinc(phi, m); }
@@ -414,10 +425,6 @@ double cephes_poch_wrap(double x, double m) { return xsf::cephes::poch(x, m); }
 
 double cephes_erfcinv(double y) { return xsf::cephes::erfcinv(y); }
 
-double cephes_exp10(double x) { return xsf::cephes::exp10(x); }
-
-double cephes_exp2(double x) { return xsf::cephes::exp2(x); }
-
 double cephes_round(double x) { return xsf::cephes::round(x); }
 
 double cephes_spence(double x) { return xsf::cephes::spence(x); }
@@ -426,6 +433,32 @@ double xsf_struve_h(double v, double z) { return xsf::struve_h(v, z); }
 
 double xsf_struve_l(double v, double z) { return xsf::struve_l(v, z); }
 
+// Exp
+
+double xsf_expm1(double x) { return xsf::expm1(x); }
+
+npy_cdouble xsf_cexpm1(npy_cdouble z) { return to_ccomplex(xsf::expm1(to_complex(z))); }
+
+double xsf_exp2(double x) { return xsf::exp2(x); }
+
+double xsf_exp10(double x) { return xsf::exp10(x); }
+
+// Log
+
+double xsf_log1p(double x) { return xsf::log1p(x); }
+
+npy_cdouble xsf_clog1p(npy_cdouble z) { return to_ccomplex(xsf::log1p(to_complex(z))); }
+
+double xsf_xlogy(double x, double y) { return xsf::xlogy(x, y); }
+
+npy_cdouble xsf_cxlogy(npy_cdouble x, npy_cdouble y) { return to_ccomplex(xsf::xlogy(to_complex(x), to_complex(y))); }
+
+double xsf_xlog1py(double x, double y) { return xsf::xlog1py(x, y); }
+
+npy_cdouble xsf_cxlog1py(npy_cdouble x, npy_cdouble y) {
+    return to_ccomplex(xsf::xlog1py(to_complex(x), to_complex(y)));
+}
+
 // Cylindrical Bessel
 
 double xsf_i0(double x) { return xsf::cyl_bessel_i0(x); }
@@ -572,7 +605,7 @@ double xsf_gdtr(double a, double b, double x) { return xsf::gdtr(a, b, x); }
 
 double xsf_gdtrc(double a, double b, double x) { return xsf::gdtrc(a, b, x); }
 
-double xsf_gdtrib(double a, double p, double x) {return xsf::gdtrib(a, p, x); }
+double xsf_gdtrib(double a, double p, double x) { return xsf::gdtrib(a, p, x); }
 
 double xsf_kolmogorov(double x) { return xsf::kolmogorov(x); }
 
@@ -592,6 +625,12 @@ double xsf_nbdtri(int k, int n, double p) { return xsf::nbdtri(k, n, p); }
 
 double xsf_ndtr(double x) { return xsf::ndtr(x); }
 
+npy_cdouble xsf_cndtr(npy_cdouble x) { return to_ccomplex(xsf::ndtr(to_complex(x))); }
+
+double xsf_log_ndtr(double x) { return xsf::log_ndtr(x); }
+
+npy_cdouble xsf_clog_ndtr(npy_cdouble x) { return to_ccomplex(xsf::log_ndtr(to_complex(x))); }
+
 double xsf_ndtri(double x) { return xsf::ndtri(x); }
 
 double xsf_owens_t(double h, double a) { return xsf::owens_t(h, a); }
