@@ -17,12 +17,9 @@
 #
 #-------------------------------------------------------------------------------
 
-# Standard library imports.
-import warnings
-
 # SciPy imports.
 from scipy import linalg, special
-from scipy._lib._util import check_random_state
+from scipy._lib._util import check_random_state, np_vecdot
 
 from numpy import (asarray, atleast_2d, reshape, zeros, newaxis, exp, pi,
                    sqrt, ravel, power, atleast_1d, squeeze, sum, transpose,
@@ -30,9 +27,8 @@ from numpy import (asarray, atleast_2d, reshape, zeros, newaxis, exp, pi,
 import numpy as np
 
 # Local imports.
-from . import _mvn
 from ._stats import gaussian_kernel_estimate, gaussian_kernel_estimate_log
-
+from ._multivariate import multivariate_normal
 
 __all__ = ['gaussian_kde']
 
@@ -53,9 +49,9 @@ class gaussian_kde:
         Datapoints to estimate from. In case of univariate data this is a 1-D
         array, otherwise a 2-D array with shape (# of dims, # of data).
     bw_method : str, scalar or callable, optional
-        The method used to calculate the estimator bandwidth.  This can be
+        The method used to calculate the bandwidth factor.  This can be
         'scott', 'silverman', a scalar constant or a callable.  If a scalar,
-        this will be used directly as `kde.factor`.  If a callable, it should
+        this will be used directly as `factor`.  If a callable, it should
         take a `gaussian_kde` instance as only parameter and return a scalar.
         If None (default), 'scott' is used.  See Notes for more details.
     weights : array_like, optional
@@ -75,12 +71,11 @@ class gaussian_kde:
 
         .. versionadded:: 1.2.0
     factor : float
-        The bandwidth factor, obtained from `kde.covariance_factor`. The square
-        of `kde.factor` multiplies the covariance matrix of the data in the kde
-        estimation.
+        The bandwidth factor obtained from `covariance_factor`.
     covariance : ndarray
-        The covariance matrix of `dataset`, scaled by the calculated bandwidth
-        (`kde.factor`).
+        The kernel covariance matrix; this is the data covariance matrix
+        multiplied by the square of the bandwidth factor, e.g.
+        ``np.cov(dataset) * factor**2``.
     inv_cov : ndarray
         The inverse of `covariance`.
 
@@ -116,7 +111,8 @@ class gaussian_kde:
         neff**(-1./(d+4)),
 
     with ``neff`` the effective number of datapoints.
-    Silverman's Rule [2]_, implemented as `silverman_factor`, is::
+    Silverman's suggestion for *multivariate* data [2]_, implemented as
+    `silverman_factor`, is::
 
         (n * (d + 2) / 4.)**(-1. / (d + 4)).
 
@@ -124,6 +120,10 @@ class gaussian_kde:
 
         (neff * (d + 2) / 4.)**(-1. / (d + 4)).
 
+    Note that this is not the same as "Silverman's rule of thumb" [6]_, which
+    may be more robust in the univariate case; see documentation of the
+    ``set_bandwidth`` method for implementing a custom bandwidth rule.
+
     Good general descriptions of kernel density estimation can be found in [1]_
     and [2]_, the mathematics for this multi-dimensional implementation can be
     found in [1]_.
@@ -137,7 +137,7 @@ class gaussian_kde:
 
     `gaussian_kde` does not currently support data that lies in a
     lower-dimensional subspace of the space in which it is expressed. For such
-    data, consider performing principle component analysis / dimensionality
+    data, consider performing principal component analysis / dimensionality
     reduction and using `gaussian_kde` with the transformed data.
 
     References
@@ -154,6 +154,8 @@ class gaussian_kde:
            Analysis, Vol. 36, pp. 279-298, 2001.
     .. [5] Gray P. G., 1969, Journal of the Royal Statistical Society.
            Series A (General), 132, 272
+    .. [6] Kernel density estimation. *Wikipedia.*
+           https://en.wikipedia.org/wiki/Kernel_density_estimation
 
     Examples
     --------
@@ -192,6 +194,16 @@ class gaussian_kde:
     >>> ax.set_ylim([ymin, ymax])
     >>> plt.show()
 
+    Compare against manual KDE at a point:
+
+    >>> point = [1, 2]
+    >>> mean = values.T
+    >>> cov = kernel.factor**2 * np.cov(values)
+    >>> X = stats.multivariate_normal(cov=cov)
+    >>> res = kernel.pdf(point)
+    >>> ref = X.pdf(point - mean).sum() / len(mean)
+    >>> np.allclose(res, ref)
+    True
     """
     def __init__(self, dataset, bw_method=None, weights=None):
         self.dataset = atleast_2d(asarray(dataset))
@@ -207,7 +219,7 @@ class gaussian_kde:
                 raise ValueError("`weights` input should be one-dimensional.")
             if len(self._weights) != self.n:
                 raise ValueError("`weights` input should be of length n")
-            self._neff = 1/sum(self._weights**2)
+            self._neff = 1/np_vecdot(self._weights, self._weights)
 
         # This can be converted to a warning once gh-10205 is resolved
         if self.d > self.n:
@@ -226,7 +238,7 @@ class gaussian_kde:
                    "of the space in which it is expressed. This has resulted "
                    "in a singular data covariance matrix, which cannot be "
                    "treated using the algorithms implemented in "
-                   "`gaussian_kde`. Consider performing principle component "
+                   "`gaussian_kde`. Consider performing principal component "
                    "analysis / dimensionality reduction and using "
                    "`gaussian_kde` with the transformed data.")
             raise linalg.LinAlgError(msg) from e
@@ -301,9 +313,9 @@ class gaussian_kde:
         cov = atleast_2d(cov)
 
         if mean.shape != (self.d,):
-            raise ValueError("mean does not have dimension %s" % self.d)
+            raise ValueError(f"mean does not have dimension {self.d}")
         if cov.shape != (self.d, self.d):
-            raise ValueError("covariance does not have dimension %s" % self.d)
+            raise ValueError(f"covariance does not have dimension {self.d}")
 
         # make mean a column vector
         mean = mean[:, newaxis]
@@ -321,8 +333,8 @@ class gaussian_kde:
         sqrt_det = np.prod(np.diagonal(sum_cov_chol[0]))
         norm_const = power(2 * pi, sum_cov.shape[0] / 2.0) * sqrt_det
 
-        energies = sum(diff * tdiff, axis=0) / 2.0
-        result = sum(exp(-energies)*self.weights, axis=0) / norm_const
+        energies = np_vecdot(diff, tdiff, axis=0) / 2.0
+        result = np_vecdot(exp(-energies), self.weights, axis=0) / norm_const
 
         return result
 
@@ -356,12 +368,11 @@ class gaussian_kde:
         normalized_low = ravel((low - self.dataset) / stdev)
         normalized_high = ravel((high - self.dataset) / stdev)
 
-        value = np.sum(self.weights*(
-                        special.ndtr(normalized_high) -
-                        special.ndtr(normalized_low)))
+        delta = special.ndtr(normalized_high) - special.ndtr(normalized_low)
+        value = np_vecdot(self.weights, delta)
         return value
 
-    def integrate_box(self, low_bounds, high_bounds, maxpts=None):
+    def integrate_box(self, low_bounds, high_bounds, maxpts=None, *, rng=None):
         """Computes the integral of a pdf over a rectangular interval.
 
         Parameters
@@ -372,6 +383,11 @@ class gaussian_kde:
             A 1-D array containing the upper bounds of integration.
         maxpts : int, optional
             The maximum number of points to use for integration.
+        rng : `numpy.random.Generator`, optional
+            Pseudorandom number generator state. When `rng` is None, a new
+            generator is created using entropy from the operating system. Types
+            other than `numpy.random.Generator` are passed to
+            `numpy.random.default_rng` to instantiate a ``Generator``.
 
         Returns
         -------
@@ -379,20 +395,12 @@ class gaussian_kde:
             The result of the integral.
 
         """
-        if maxpts is not None:
-            extra_kwds = {'maxpts': maxpts}
-        else:
-            extra_kwds = {}
-
-        value, inform = _mvn.mvnun_weighted(low_bounds, high_bounds,
-                                            self.dataset, self.weights,
-                                            self.covariance, **extra_kwds)
-        if inform:
-            msg = ('An integral in _mvn.mvnun requires more points than %s' %
-                   (self.d * 1000))
-            warnings.warn(msg)
-
-        return value
+        low, high = low_bounds - self.dataset.T, high_bounds - self.dataset.T
+        values = multivariate_normal.cdf(
+            high, lower_limit=low, cov=self.covariance, maxpts=maxpts,
+            rng=rng
+        )
+        return np_vecdot(values, self.weights, axis=-1)
 
     def integrate_kde(self, other):
         """
@@ -434,8 +442,8 @@ class gaussian_kde:
             diff = large.dataset - mean
             tdiff = linalg.cho_solve(sum_cov_chol, diff)
 
-            energies = sum(diff * tdiff, axis=0) / 2.0
-            result += sum(exp(-energies)*large.weights, axis=0)*small.weights[i]
+            energies = np_vecdot(diff, tdiff, axis=0) / 2.0
+            result += np_vecdot(exp(-energies), large.weights, axis=0)*small.weights[i]
 
         sqrt_det = np.prod(np.diagonal(sum_cov_chol[0]))
         norm_const = power(2 * pi, sum_cov.shape[0] / 2.0) * sqrt_det
@@ -466,7 +474,7 @@ class gaussian_kde:
         resample : (self.d, `size`) ndarray
             The sampled dataset.
 
-        """
+        """ # numpy/numpydoc#87  # noqa: E501
         if size is None:
             size = int(self.neff)
 
@@ -501,14 +509,13 @@ class gaussian_kde:
 
     #  Default method to calculate bandwidth, can be overwritten by subclass
     covariance_factor = scotts_factor
-    covariance_factor.__doc__ = """Computes the coefficient (`kde.factor`) that
-        multiplies the data covariance matrix to obtain the kernel covariance
-        matrix. The default is `scotts_factor`.  A subclass can overwrite this
+    covariance_factor.__doc__ = """Computes the bandwidth factor `factor`.
+        The default is `scotts_factor`.  A subclass can overwrite this
         method to provide a different method, or set it through a call to
-        `kde.set_bandwidth`."""
+        `set_bandwidth`."""
 
     def set_bandwidth(self, bw_method=None):
-        """Compute the estimator bandwidth with given method.
+        """Compute the bandwidth factor with given method.
 
         The new bandwidth calculated after a call to `set_bandwidth` is used
         for subsequent evaluations of the estimated density.
@@ -516,12 +523,12 @@ class gaussian_kde:
         Parameters
         ----------
         bw_method : str, scalar or callable, optional
-            The method used to calculate the estimator bandwidth.  This can be
+            The method used to calculate the bandwidth factor.  This can be
             'scott', 'silverman', a scalar constant or a callable.  If a
-            scalar, this will be used directly as `kde.factor`.  If a callable,
+            scalar, this will be used directly as `factor`.  If a callable,
             it should take a `gaussian_kde` instance as only parameter and
             return a scalar.  If None (default), nothing happens; the current
-            `kde.covariance_factor` method is kept.
+            `covariance_factor` method is kept.
 
         Notes
         -----
@@ -698,7 +705,7 @@ class gaussian_kde:
         try:
             return self._neff
         except AttributeError:
-            self._neff = 1/sum(self.weights**2)
+            self._neff = 1/np_vecdot(self.weights, self.weights)
             return self._neff
 
 
