@@ -54,7 +54,7 @@ def main():
             name = NAME_MAP.get(name, name)
             if disp:
                 if name not in names:
-                    stdout_b.write(("    - Author: %s\n" % name).encode('utf-8'))
+                    stdout_b.write((f"    - Author: {name}\n").encode())
             names.update((name,))
 
         # Look for "thanks to" messages in the commit log
@@ -68,7 +68,7 @@ def main():
             name = m.group(2)
             if name not in ('this',):
                 if disp:
-                    stdout_b.write("    - Log   : %s\n" % line.strip().encode('utf-8'))
+                    stdout_b.write(f"    - Log   : {line.strip().encode()}\n")
                 name = NAME_MAP.get(name, name)
                 names.update((name,))
 
@@ -111,7 +111,7 @@ def main():
         # Print some empty lines to separate
         stdout_b.write(b"\n\n")
         for author in n_authors:
-            stdout_b.write(("- %s\n" % author).encode('utf-8'))
+            stdout_b.write((f"- {author}\n").encode())
         # return for early exit so we only print new authors
         return
 
@@ -138,12 +138,11 @@ Authors
         else:
             stdout_b.write((f"* {author_clean} ({count}) +\n").encode())
 
-    stdout_b.write(("""
-A total of %(count)d people contributed to this release.
-People with a "+" by their names contributed a patch for the first time.
-This list of names is automatically generated, and may not be fully complete.
-
-""" % dict(count=len(authors))).encode('utf-8'))
+    stdout_b.write((f"""
+    A total of {len(authors)} people contributed to this release.
+    People with a "+" by their names contributed a patch for the first time.
+    This list of names is automatically generated, and may not be fully complete.
+    """).encode())
 
     stdout_b.write(b"\nNOTE: Check this list manually! It is automatically generated "
                    b"and some names\n      may be missing.\n")
@@ -202,7 +201,7 @@ class Cmd:
     def __call__(self, command, *a, **kw):
         ret = self._call(command, a, {}, call=True, **kw)
         if ret != 0:
-            raise RuntimeError("%s failed" % self.executable)
+            raise RuntimeError(f"{self.executable} failed")
 
     def pipe(self, command, *a, **kw):
         stdin = kw.pop('stdin', None)
@@ -215,7 +214,7 @@ class Cmd:
                       call=False, **kw)
         out, err = p.communicate()
         if p.returncode != 0:
-            raise RuntimeError("%s failed" % self.executable)
+            raise RuntimeError(f"{self.executable} failed")
         return out
 
     def readlines(self, command, *a, **kw):
