@@ -102,12 +102,10 @@ REFGUIDE_AUTOSUMMARY_SKIPLIST = [
     r'scipy\.special\.jn',  # alias for jv
     r'scipy\.ndimage\.sum',   # alias for sum_labels
     r'scipy\.linalg\.solve_lyapunov',  # deprecated name
-    r'scipy\.signal\.sosfreqz',  # alias for freqz_sos
     r'scipy\.stats\.contingency\.chi2_contingency',
     r'scipy\.stats\.contingency\.expected_freq',
     r'scipy\.stats\.contingency\.margins',
     r'scipy\.stats\.reciprocal',  # alias for lognormal
-    r'scipy\.stats\.trapz',   # alias for trapezoid
 ]
 # deprecated windows in scipy.signal namespace
 for name in ('barthann', 'bartlett', 'blackmanharris', 'blackman', 'bohman',
@@ -251,8 +249,8 @@ def check_items(all_dict, names, deprecated, others, module_name, dots=True):
 
     output = ""
 
-    output += "Non-deprecated objects in __all__: %i\n" % num_all
-    output += "Objects in refguide: %i\n\n" % num_ref
+    output += f"Non-deprecated objects in __all__: {num_all}\n"
+    output += f"Objects in refguide: {num_ref}\n\n"
 
     only_all, only_ref, missing = compare(all_dict, others, names, module_name)
     dep_in_ref = only_ref.intersection(deprecated)
@@ -310,7 +308,7 @@ def validate_rst_syntax(text, name, dots=True):
         'obj', 'versionadded', 'versionchanged', 'module', 'class', 'meth',
         'ref', 'func', 'toctree', 'moduleauthor', 'deprecated',
         'sectionauthor', 'codeauthor', 'eq', 'doi', 'DOI', 'arXiv', 'arxiv',
-        'versionremoved',
+        'versionremoved', 'math:numref'
     ])
 
     # Run through docutils
@@ -368,7 +366,7 @@ def validate_rst_syntax(text, name, dots=True):
     if not success:
         output += "    " + "-"*72 + "\n"
         for lineno, line in enumerate(text.splitlines()):
-            output += "    %-4d    %s\n" % (lineno+1, line)
+            output += f"    {lineno+1:<4d}    {line}\n"
         output += "    " + "-"*72 + "\n\n"
 
     if dots:
@@ -537,7 +535,7 @@ def main(argv):
     success = True
     results = []
 
-    print("Running checks for %d modules:" % (len(modules),))
+    print(f"Running checks for {len(modules)} modules:")
 
     for module in modules:
         if dots:
